import { Query as AttestationQuery } from '@routerprotocol/chain-api/attestation/query_pb_service';
import {
  QueryLatestValsetNonceRequest,
  QueryLatestValsetNonceResponse,
  QueryAllValsetRequest,
  QueryAllValsetResponse,
  QueryGetValsetResponse,
  QueryGetValsetRequest,
  QueryLatestValsetRequest,
  QueryLatestValsetResponse,
  QueryLastEventNonceRequest,
  QueryLastEventNonceResponse,
  QueryListOrchestratorsRequest,
  QueryListOrchestratorsResponse
} from '@routerprotocol/chain-api/attestation/query_pb';
import BaseConsumer from './BaseGrpcConsumer';
import { AttestationTransformer } from '../transformers';

/**
 * @category Chain Grpc API
 */
export class ChainGrpcAttestationApi extends BaseConsumer {

  async fetchLatestValsetNonce() {
    const request = new QueryLatestValsetNonceRequest();

    try {
      const response = await this.request<
        QueryLatestValsetNonceRequest,
        QueryLatestValsetNonceResponse,
        typeof AttestationQuery.LatestValsetNonce
      >(request, AttestationQuery.LatestValsetNonce);
      
      return AttestationTransformer.latestValsetNonce(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }

  async fetchAllValsets() {
    const request = new QueryAllValsetRequest();

    try {
      const response = await this.request<
        QueryAllValsetRequest,
        QueryAllValsetResponse,
        typeof AttestationQuery.ValsetAll
      >(request, AttestationQuery.ValsetAll);

      return AttestationTransformer.allValset(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }

  async fetchValsetByNonce(valsetNonce: number) {
    const request = new QueryGetValsetRequest();
    request.setNonce(valsetNonce);

    try {
      const response = await this.request<
        QueryGetValsetRequest,
        QueryGetValsetResponse,
        typeof AttestationQuery.Valset
      >(request, AttestationQuery.Valset);

      return AttestationTransformer.valsetByNonce(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }

  async fetchLatestValset() {
    const request = new QueryLatestValsetRequest();

    try {
      const response = await this.request<
        QueryLatestValsetRequest,
        QueryLatestValsetResponse,
        typeof AttestationQuery.LatestValset
      >(request, AttestationQuery.LatestValset);

      return AttestationTransformer.latestValset(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }

  async fetchLastEventByValidator(chainType: number, chainId: string, valAddress: string) {
    const request = new QueryLastEventNonceRequest();
    request.setChainType(chainType);
    request.setChainId(chainId);
    request.setValidatorAddress(valAddress);

    try {
      const response = await this.request<
        QueryLastEventNonceRequest,
        QueryLastEventNonceResponse,
        typeof AttestationQuery.LastEventNonce
      >(request, AttestationQuery.LastEventNonce);

      return AttestationTransformer.lastEventByValidator(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }

  async fetchAllOrchestrators() {
    const request = new QueryListOrchestratorsRequest();

    try {
      const response = await this.request<
      QueryListOrchestratorsRequest,
      QueryListOrchestratorsResponse,
        typeof AttestationQuery.ListOrchestrators
      >(request, AttestationQuery.ListOrchestrators);

      return AttestationTransformer.listOrchestrators(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }
}
