import { Query as BankQuery } from '@routerprotocol/chain-api/cosmos/bank/v1beta1/query_pb_service';
import {
  QueryAllBalancesRequest,
  QueryAllBalancesResponse,
  QueryBalanceRequest,
  QueryBalanceResponse
} from '@routerprotocol/chain-api/cosmos/bank/v1beta1/query_pb';
import BaseConsumer from './BaseGrpcConsumer';
import { BankTransformer } from '../transformers';

/**
 * @category Chain Grpc API
 */
export class ChainGrpcBankApi extends BaseConsumer {

  async fetchBalance({
    accountAddress,
    denom,
  }: {
    accountAddress: string;
    denom: string;
  }) {
    const request = new QueryBalanceRequest();
    request.setAddress(accountAddress);
    request.setDenom(denom);

    try {
      const response = await this.request<
        QueryBalanceRequest,
        QueryBalanceResponse,
        typeof BankQuery.Balance
      >(request, BankQuery.Balance);

      return BankTransformer.balanceResponseToBalance(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }

  async fetchBalances(address: string) {
    const request = new QueryAllBalancesRequest();
    request.setAddress(address);

    try {
      const response = await this.request<
        QueryAllBalancesRequest,
        QueryAllBalancesResponse,
        typeof BankQuery.AllBalances
      >(request, BankQuery.AllBalances);

      return BankTransformer.balancesResponseToBalances(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }
}
