import { Query as MultiChainQuery } from '@routerprotocol/chain-api/multichain/query_pb_service';
import { 
    QueryAllChainConfigRequest,
    QueryAllChainConfigResponse, 
    QueryGetChainConfigRequest, 
    QueryGetChainConfigResponse
 } from '@routerprotocol/chain-api/multichain/query_pb';
import BaseConsumer from './BaseGrpcConsumer';
import { MultiChainTransformer } from '../transformers';

export class ChainGrpcMultiChainApi extends BaseConsumer {

    async fetchChainConfig(chainType: number, chainId: string) {
        const request = new QueryGetChainConfigRequest();
        request.setChainType(chainType);
        request.setChainId(chainId);

        try {
            const response = await this.request<
            QueryGetChainConfigRequest,
            QueryGetChainConfigResponse,
            typeof MultiChainQuery.ChainConfig
            >(request, MultiChainQuery.ChainConfig);

            console.log("api call success")

            return MultiChainTransformer.chainConfig(response);
        } catch (e) {
            //@ts-ignore
            throw new Error(e.message);
        }
    }

    async fetchAllChainConfig() {
        const request = new QueryAllChainConfigRequest();

        try {
            const response = await this.request<
            QueryAllChainConfigRequest,
            QueryAllChainConfigResponse,
            typeof MultiChainQuery.ChainConfigAll
            >(request, MultiChainQuery.ChainConfigAll);

            return MultiChainTransformer.allChainConfig(response);
        } catch (e) {
            //@ts-ignore
            throw new Error(e.message);
        }
    }
}