import { Query as OutboundQuery } from '@routerprotocol/chain-api/outbound/query_pb_service';
import {
    QueryAllOutgoingBatchTxRequest,
    QueryAllOutgoingBatchTxResponse,
    QueryGetOutgoingBatchTxRequest,
    QueryGetOutgoingBatchTxResponse,
    QueryGetOutgoingBatchConfirmRequest,
    QueryGetOutgoingBatchConfirmResponse,
    QueryAllOutgoingBatchConfirmRequest,
    QueryAllOutgoingBatchConfirmResponse,
    QueryLastOutboundBatchNonceRequest,
    QueryLastOutboundBatchNonceResponse
} from '@routerprotocol/chain-api/outbound/query_pb';
import BaseConsumer from './BaseGrpcConsumer';
import { OutboundTransformer } from '../transformers';

/**
 * @category Chain Grpc API
 */
export class ChainGrpcOutboundApi extends BaseConsumer {

  async fetchAllOutgoingBatchTx() {
    const request = new QueryAllOutgoingBatchTxRequest();

    try {
      const response = await this.request<
        QueryAllOutgoingBatchTxRequest,
        QueryAllOutgoingBatchTxResponse,
        typeof OutboundQuery.OutgoingBatchTxAll
      >(request, OutboundQuery.OutgoingBatchTxAll);

      return OutboundTransformer.allOutgoingBatchTx(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }

  async fetchOutgoingBatchTx(
    destinationChainType: number,
    destinationChainId: string,
    sourceAddress: string,
    batchNonce: number
  ) {
    const request = new QueryGetOutgoingBatchTxRequest();
    request.setDestinationChainType(destinationChainType);
    request.setDestinationChainId(destinationChainId);
    request.setSourceAddress(sourceAddress);
    request.setNonce(batchNonce);

    try {
      const response = await this.request<
        QueryGetOutgoingBatchTxRequest,
        QueryGetOutgoingBatchTxResponse,
        typeof OutboundQuery.OutgoingBatchTx
      >(request, OutboundQuery.OutgoingBatchTx);

      return OutboundTransformer.outgoingBatchTx(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }

  async fetchOutgoingBatchTxConfirm(
    destinationChainType: number,
    destinationChainId: string,
    sourceAddress: string,
    batchNonce: number,
    orchestrator: string
  ) {
    const request = new QueryGetOutgoingBatchConfirmRequest();
    request.setDestinationChainType(destinationChainType);
    request.setDestinationChainId(destinationChainId);
    request.setSourceAddress(sourceAddress);
    request.setNonce(batchNonce);
    request.setOrchestrator(orchestrator);

    try {
      const response = await this.request<
      QueryGetOutgoingBatchConfirmRequest,
        QueryGetOutgoingBatchConfirmResponse,
        typeof OutboundQuery.OutgoingBatchConfirm
      >(request, OutboundQuery.OutgoingBatchConfirm);

      return OutboundTransformer.outgoingBatchTxConfirm(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }

  async fetchAllOutgoingBatchTxConfirms(
    destinationChainType: number,
    destinationChainId: string,
    sourceAddress: string,
    batchNonce: number
  ) {
    const request = new QueryAllOutgoingBatchConfirmRequest();
    request.setDestinationChainType(destinationChainType);
    request.setDestinationChainId(destinationChainId);
    request.setSourceAddress(sourceAddress);
    request.setNonce(batchNonce);

    try {
      const response = await this.request<
      QueryAllOutgoingBatchConfirmRequest,
        QueryAllOutgoingBatchConfirmResponse,
        typeof OutboundQuery.OutgoingBatchConfirmAll
      >(request, OutboundQuery.OutgoingBatchConfirmAll);

      return OutboundTransformer.allOutgoingBatchTxConfirms(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }

  async fetchLastOutboundBatchNonce(destinationChainType: number, destinationChainId: string, sourceAddress: string) {
    const request = new QueryLastOutboundBatchNonceRequest();
    request.setDestinationChainType(destinationChainType);
    request.setDestinationChainId(destinationChainId);
    request.setSourceAddress(sourceAddress);

    try {
      const response = await this.request<
        QueryLastOutboundBatchNonceRequest,
        QueryLastOutboundBatchNonceResponse,
        typeof OutboundQuery.LastOutboundBatchNonce
      >(request, OutboundQuery.LastOutboundBatchNonce);

      return OutboundTransformer.lastOutboundBatchNonce(response);
    } catch (e) {
      //@ts-ignore
      throw new Error(e.message);
    }
  }
}
