import { Query as WasmQuery } from '@routerprotocol/chain-api/cosmwasm/wasm/v1/query_pb_service';
import {
    QuerySmartContractStateRequest,
    QuerySmartContractStateResponse,
    QueryRawContractStateRequest,
    QueryRawContractStateResponse
} from '@routerprotocol/chain-api/cosmwasm/wasm/v1/query_pb';
import BaseConsumer from './BaseGrpcConsumer';
import { WasmTransformer } from '../transformers';

/**
 * @category Chain Grpc API
 */
export class ChainGrpcWasmApi extends BaseConsumer {

    async smartContractState(
        contractAddress: string,
        queryData: string
    ) {
        const request = new QuerySmartContractStateRequest();
        request.setAddress(contractAddress);
        request.setQueryData(queryData);

        try {
            const response = await this.request<
                QuerySmartContractStateRequest,
                QuerySmartContractStateResponse,
                typeof WasmQuery.SmartContractState
            >(request, WasmQuery.SmartContractState);

            return WasmTransformer.smartContractState(response);
        } catch (e) {
            //@ts-ignore
            throw new Error(e.message);
        }
    }

    async rawContractState(
        contractAddress: string,
        queryData: string
    ) {
        const request = new QueryRawContractStateRequest();
        request.setAddress(contractAddress);
        request.setQueryData(queryData);

        try {
            const response = await this.request<
                QueryRawContractStateRequest,
                QueryRawContractStateResponse,
                typeof WasmQuery.RawContractState
            >(request, WasmQuery.RawContractState);

            return WasmTransformer.rawContractState(response);
        } catch (e) {
            //@ts-ignore
            throw new Error(e.message);
        }
    }
}
